#property copyright "Naddmer" 
#property indicator_chart_window 

#property indicator_buffers 2
#property indicator_color1 Red
#property indicator_color2 Yellow
#property indicator_width2 1
//
#include <common_functions.mqh>
#include <mt4R.mqh>
// Eventhandler
#include <eventhandler.mqh>

extern string RTermPath_desc="Commandline to call RTERM.EXE";
extern string RTermPath="C:\Apps\R\R-3.0.3\bin\i386\Rterm.exe --vanilla --no-readline";
//
extern bool saveWS=false;
#define PM_OPEN 0
#define PM_HIGH 1
#define PM_LOW 2
#define PM_CLOSE 3
#define PM_TYPICAL 4
#define PM_MEDIAN 5
#define PM_WEIGHTED 6
extern string priceModel_desc1="Open=0, High=1, Low=2, Close=3,";
extern string priceModel_desc2="Typical=4, Median=5, Weighted=6";
extern string priceModel_desc3="Default: 4 (Typical)";
extern int priceModel=4;
extern string RWorkPath_desc="Where are the R-Snapshots to be saved?";
extern string RWorkPath="C:/RData";
//
extern string back_desc = "How many samples for the prediction?";
extern int sampleCount = 1000;
int realSampleCount = 0;
//
string foreCount_desc1="How many predictions?";
string foreCount_desc2="(Note: Each one takes 10 secs. to complete!)";
extern int foreCount=1;
//
extern string pipFactor_desc="Multiply logrets with this factor. 0==>10^Digits";
extern double pipFactor=0;

//
static int curPeriod=0;
static bool initiallySaved=false;
//
int R;
// double hist_r[][6];
MqlRates hist_r[];
double buf_upper[];
double buf_lower[];
datetime lastBarTime=0;

string 	CR="\r\n";
string PREDICTION_COMMAND;

int init(){
	SetIndexArrow(0, 119);
	SetIndexBuffer(0, buf_upper);
	SetIndexStyle(0,DRAW_ARROW,STYLE_DOT,1,LightGreen);
	//
	SetIndexArrow(1, 119);
	SetIndexBuffer(1, buf_lower);
	SetIndexStyle(1,DRAW_ARROW,STYLE_DOT,1,Orange);
	if (pipFactor==0) {
		pipFactor=MathExp(Digits*MathLog(10));
	}
	//
	curPeriod=Period();
	R = RInit(RTermPath, 2);
	//
	// Assign const values ...
	RAssignInteger(R, "PM_OPEN", PM_OPEN);
	RAssignInteger(R, "PM_HIGH", PM_HIGH);
	RAssignInteger(R, "PM_LOW", PM_LOW);
	RAssignInteger(R, "PM_CLOSE", PM_CLOSE);
	RAssignInteger(R, "PM_TYPICAL", PM_TYPICAL);
	RAssignInteger(R, "PM_MEDIAN", PM_MEDIAN);
	RAssignInteger(R, "PM_WEIGHTED", PM_WEIGHTED);
	//
	RAssignDouble( R, "pipFactor", pipFactor);
	RAssignInteger(R, "foreCount", foreCount);
	RAssignInteger(R, "priceModel", priceModel);
	
	RExecute(R, 
		"pkgs <- c('zoo', 'xts', 'fBasics', 'MASS', 'quantmod', 'fGarch')" + CR
		+ "lapply(pkgs, require, character.only=T)" + CR
		+ "getHist <- function(z) {" + CR
		+ "  if (priceModel==PM_OPEN) { h <- z$Open } " + CR
		+ "  if (priceModel==PM_HIGH) { h <- z$High } " + CR
		+ "  if (priceModel==PM_LOW) { h <- z$Low } " + CR
		+ "  if (priceModel==PM_CLOSE) { h <- z$Close } " + CR
		+ "  if (priceModel==PM_TYPICAL) { h <- (z$High + z$Low + z$Close) / 3 } " + CR
		+ "  if (priceModel==PM_MEDIAN) { h <- (z$High + z$Low) / 2 } " + CR
		+ "  if (priceModel==PM_WEIGHTED) { h <- (z$High + z$Low + 2*z$Close) / 4 } " + CR
		+ "  return(h)" + CR
		+ "}"
	);
	
	PREDICTION_COMMAND=
		  "z <- head(tail(hist_r, sampleCount+foreCount+2), sampleCount+foreCount)" + CR
		+ "z <- as.xts(z[,2:6], order.by=as.POSIXct(z$Timestamp, origin='1970-01-01 00:00', tz='UTC'))" + CR
		+ "hist <- getHist(z) " + CR
		+ "rets <- diff(pipFactor*log(hist))" + CR
		+ "forecasts <- c()" + CR
		+ "histSize <- dim(z)[0]" + CR
		+ "retSize <- length(rets)-1" + CR
		+ "beginIdx <- (retSize - foreCount + 1):(retSize)" + CR
		+ "for (idx in beginIdx) " + CR 
		+ "{"  + CR
		+ "   d <- rets[(idx-sampleCount+2):(idx)]" + CR
		+ "   forecasts <- c(forecasts, predict("  + CR
		+ "      garchFit(~arma(0, 2) + garch(1, 1), " + CR
		+ "        data=d, trace=FALSE), " + CR
		+ "        n.ahead=1, doplot=F)[,1])"  + CR
		+"}"  + CR
	;
	
	ArrayCopyRates(hist_r, Symbol(), 0);

	// Initialize Eventhandler to provide onBar and onTick Events.
	events_initEvents();
	doRDraw();
	return(0);
}

int deinit(){
	Comment("");
	RDeinit(R);
	return(0);
}

int start(){
	// Perform eventhandler calls.
	events_doEvents();
	return(0);   
}

void events_onBar(int pPeriod) {
	switch (pPeriod) {
	case PERIOD_M1:
		Comment("$Rev: 596 $" 
			+ " history: " + realSampleCount + " bars" 
			+ ", Datapath: " + RWorkPath
		);
		Print("onBar: working on M1...");
		doRDraw();
		break;
	case PERIOD_M5:
		// Print("onBar: working on M5...");
		if (saveWS) {
			RExecute(R, "setwd('" + RWorkPath + "')");
			RExecute(R,"save.image(file='"+ createFileName() +"')");
		}
		break;
	case PERIOD_M15:
		break;
	case PERIOD_M30:
		break;
	case PERIOD_H1:
		break;
	case PERIOD_H4:
		break;
	case PERIOD_D1:
		break;
	case PERIOD_W1:
		break;
	default:
		break;
	}
	return;
}

void events_onTick() {
	return;
}

void doRDraw() {
	if (!RIsRunning(R)) {
		return;
	}
	DisplayComment();
	//
	// Hier stehen die Vorhersagen...
	double forecasts[];
	ArrayResize(forecasts, foreCount);
	int i;
	// Umschalten von Timeframes macht komische Sachen! :-)
	if (Period()!=curPeriod) {
		curPeriod=Period();
		doRDraw();
	}
	
	if (Bars<(sampleCount + foreCount) ) {
		realSampleCount=Bars;
	} else {
		realSampleCount=sampleCount;
	}
	RAssignInteger(R, "sampleCount", realSampleCount);
	
	// Kaut immer noch auf den Zahlen von vorhin herum ... :-)
	if (RIsBusy(R)) {
		return;
	}
	if (RGetInteger(R, "exists('forecasts')")==1 ) {
		int k;
		k=RGetInteger(R, "length(forecasts)");
		if (k>0 )  {
			// Hier werden ggf, mehrere Zeilen an Echo erzeugt. 
			DbgLog("RExecute finished - getting vector vSize=", k);
			k=RGetVector(R, "forecasts", forecasts, foreCount);
			DbgLog("RExecute finished - got vector with size", k);
			int j=0;
			for (i=0; i<k; i++){
				j=k - i - 1;
				if (forecasts[j]<0) {
					buf_lower[i] = Open[i];
				} else if (forecasts[j]>0) {
					buf_upper[i] = Open[i];
				}
				Print("doRDraw: "
					+ "Forecast for " + TimeToStr(iTime(NULL, 0, i), TIME_DATE|TIME_MINUTES|TIME_SECONDS) 
					+ " is: " + DoubleToStr( forecasts[j], 3)
				);
			}
			DbgLog("got values from vector: ", k);
		}
		// RExecute(R, "remove(forecasts)");
	}
	// make a (new) prediction
	// move some history over to R   
	
	RAssignOHLC(R, "hist_r", 	hist_r, Bars);
	if (saveWS && !initiallySaved) {
		initiallySaved=true;
		RExecute(R, "setwd('" + RWorkPath + "')");
		RExecute(R,"save.image(file='"+ createFileName() + "')");
	}
	// return;
	RExecuteAsync(R, PREDICTION_COMMAND);
	// RExecute(R, PREDICTION_COMMAND);
}

/**
creates a filename for the RData export file.
*/
string createFileName() {
	string name=
		WindowExpertName() + "_" + 
		Symbol() + "_" + 
		Period() + 
		".RData";
	return(name);
}

void DisplayComment() {
	Comment("$Rev: 596 $" 
		+ " history: " + realSampleCount + " bars" 
		+ ", Datapath: " + RWorkPath,
		"\n lastBarTime: " + TimeToStr(lastBarTime, TIME_DATE|TIME_MINUTES)
		
	);
}